# -*- coding: utf-8 -*-


class BaseError(Exception):
    """
    Baseclass for all Holiday errors.
    """
    pass


class ParseError(BaseError):
    """
    Holiday dates value parse error.
    """


class PeriodRangeError(BaseError):
    """
    Outside the scope of the period error.
    """



# -*- coding: utf-8 -*-

"""
    holiday
    ~~~~~~~
    :copyright: (c) 2016 by wanshot.
    :license: MIT, see LICENSE for more details.

    SpecialThanks
    ~~~~~~~~~~~~~
    @crohaco: He gave holiday idea to me and he did a review
    @tell-k: He taught me pypi debut way

"""

__version__ = '1.0.0'
__license__ = 'MIT License',
__author__ = 'wanshot'
__author_email__ = 'nishikawa0228@sj9.so-net.ne.jp'


from .core import Holiday
from .exceptions import ParseError, PeriodRangeError
from ._compat import string_types



# -*- coding: utf-8 -*-

import unittest
from datetime import date
from holiday.exceptions import (
    ParseError,
    PeriodRangeError,
)


class TestHoliday(unittest.TestCase):
    """ Test Command
    python -m unittest discover
    """

    def _getTargetFunc(self):
        from holiday.core import Holiday
        return Holiday

    def _makeOne(self):
        return self._getTargetFunc()

    def test_with_ok_map_day_of_week(self):

        mon = self._makeOne()([(2015, 12, 28, "mon", 5)])
        tue = self._makeOne()([(2015, 12, 29, "tue", 5)])
        wed = self._makeOne()([(2015, 12, 30, "wed", 5)])
        thu = self._makeOne()([(2015, 12, 31, "thu", 5)])
        fri = self._makeOne()([(2016, 1, 1, "fri", 1)])
        sat = self._makeOne()([(2016, 1, 2, "sat", 1)])
        sun = self._makeOne()([(2016, 1, 3, "sun", 1)])
        self.assertTrue(mon.is_holiday(date(2015, 12, 28)))
        self.assertTrue(tue.is_holiday(date(2015, 12, 29)))
        self.assertTrue(wed.is_holiday(date(2015, 12, 30)))
        self.assertTrue(thu.is_holiday(date(2015, 12, 31)))
        self.assertTrue(fri.is_holiday(date(2016, 1, 1)))
        self.assertTrue(sat.is_holiday(date(2016, 1, 2)))
        self.assertTrue(sun.is_holiday(date(2016, 1, 3)))

    def test_with_ok_asterisk_year(self):
        asterisk = self._makeOne()([("*", 1, 1, "fri", 1)])
        self.assertTrue(asterisk.is_holiday(date(2016, 1, 1)))

    def test_with_ok_asterisk_month(self):
        asterisk = self._makeOne()([(2016, "*", 1, "fri", 1)])
        self.assertTrue(asterisk.is_holiday(date(2016, 1, 1)))

    def test_with_ok_asterisk_day(self):
        asterisk = self._makeOne()([(2016, 1, "*", "fri", 1)])
        self.assertTrue(asterisk.is_holiday(date(2016, 1, 1)))

    def test_with_ok_asterisk_day_of_week(self):
        asterisk = self._makeOne()([(2016, 1, 1, "*", 1)])
        self.assertTrue(asterisk.is_holiday(date(2016, 1, 1)))

    def test_with_ok_asterisk_num_of_week(self):
        asterisk = self._makeOne()([(2016, 1, 1, "fri", "*")])
        self.assertTrue(asterisk.is_holiday(date(2016, 1, 1)))

    def test_with_ng_exclude_asterisk_and_int_year(self):
        ng_values = [u"×", "test", 1.234]
        for ng in ng_values:
            self.assertRaises(TypeError, lambda: self._makeOne()([(ng, 1, 1, "fri", 1)]))

    def test_with_ng_exclude_asterisk_and_int_month(self):
        ng_values = [u"×", "test", 1.234]
        for ng in ng_values:
            self.assertRaises(TypeError, lambda: self._makeOne()([(2016, ng, 1, "fri", 1)]))

    def test_with_ng_exclude_asterisk_and_int_day(self):
        ng_values = [u"×", "test", 1.234]
        for ng in ng_values:
            self.assertRaises(TypeError, lambda: self._makeOne()([(2016, 1, ng, "fri", 1)]))

    def test_with_ng_exclude_asterisk_and_day_of_week_name(self):
        ng_char_values = [u"×", "test"]
        for ng in ng_char_values:
            self.assertRaises(ParseError, lambda: self._makeOne()([(2016, 1, 1, ng, 1)]))

    def test_with_ng_float_day_of_week(self):
        self.assertRaises(TypeError, lambda: self._makeOne()([(2016, 1, 1, 1.234, 1)]))

    def test_with_ng_exclude_asterisk_and_int_week_of_num(self):
        ng_char_values = [u"×", "test", 1.234]
        for ng in ng_char_values:
            self.assertRaises(TypeError, lambda: self._makeOne()([(2016, 1, 1, "fri", ng)]))

    def test_with_ok_is_business_day(self):
        asterisk = self._makeOne()([(2016, 1, 1, "fri", 1)])
        self.assertFalse(asterisk.is_business_day(date(2016, 1, 1)))

    def test_with_ng_is_business_day(self):
        asterisk = self._makeOne()([(2016, 1, 1, "fri", 1)])
        self.assertTrue(asterisk.is_business_day(date(2016, 1, 2)))

    def test_with_ok_exception_holiday(self):
        self.assertRaises(ParseError, lambda: self._makeOne()([(2016, 1, 1, "test", 1)]))
        self.assertRaises(PeriodRangeError, lambda: self._makeOne()([(10000, 1, 1, "fri", 1)]))
        self.assertRaises(PeriodRangeError, lambda: self._makeOne()([(2016, 13, 1, "fri", 1)]))
        self.assertRaises(PeriodRangeError, lambda: self._makeOne()([(2016, 1, 32, "fri", 1)]))
        self.assertRaises(PeriodRangeError, lambda: self._makeOne()([(2016, 1, 1, 8, 1)]))
        self.assertRaises(PeriodRangeError, lambda: self._makeOne()([(2016, 1, 32, "fri", 6)]))


def test():
    suite = unittest.TestLoader().loadTestsFromTestCase(TestHoliday)
    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__':
    test()



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
from holiday import (
    __version__,
    __license__,
    __author__,
    __author_email__,
)

__name__ = 'holiday'
__url__ = 'https://github.com/wanshot/holiday'

__short_description__ = __name__ + ' is a package to generate holiday.'
__long_description__ = open('./README.rst', 'r').read()

__classifiers__ = [
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Development Status :: 5 - Production/Stable',
    'Topic :: Software Development',
    'Programming Language :: Python :: 2',
]

__keywords__ = [
    'datetime',
    'date',
    'time',
    'calendar',
]

setup(
    name=__name__,
    version=__version__,
    description=__short_description__,
    long_description=__long_description__,
    url=__url__,
    author=__author__,
    author_email=__author_email__,
    classifiers=__classifiers__,
    keywords=' ,'.join(__keywords__),
    license=__license__,
    packages=find_packages(exclude=['tests']),
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from collections import (
    defaultdict,
    OrderedDict,
)
from itertools import product

from .exceptions import (
    ParseError,
    PeriodRangeError
)
from ._compat import string_types


WEEK_MAP = OrderedDict((
    ("mon", 1),
    ("tue", 2),
    ("wed", 3),
    ("thu", 4),
    ("fri", 5),
    ("sat", 6),
    ("sun", 7),
))

ORDER_WEEK = WEEK_MAP.keys()

TIME_INFO = OrderedDict((
    ("year", (1, 9999)),
    ("month", (1, 12)),
    ("day", (1, 31)),
    ("day_of_week", (1, 7)),
    ("num_of_week", (1, 6)),
))

TIME_LABEL = TIME_INFO.keys()


class Holiday(object):
    """
    Base class
    """

    def __init__(self, times):
        """
        :param tuple of list times: plz see below
        """
        # ('*', '*', '*', '*', '*')
        #   ┬    ┬    ┬    ┬    ┬
        #   │    │    │    │    │
        #   │    │    │    │    │
        #   │    │    │    │    └─  number of week (1 - 5)
        #   │    │    │    └─── day of week (1 to 7 or mon to sun)
        #   │    │    └───── day of month (1 - 31)
        #   │    └─────── month (1 - 12)
        #   └───────── year (1 - 9999)

        self._check_holiday_structure(times)

        self.year = defaultdict(set)
        self.month = defaultdict(set)
        self.day = defaultdict(set)
        self.day_of_week = defaultdict(set)
        self.num_of_week = defaultdict(set)

        for idx, (year, month, day, day_of_week, num_of_week) in enumerate(times):
            if isinstance(day_of_week, string_types):
                day_of_week = WEEK_MAP.get(day_of_week, "*")
            self.year[year].add(idx)
            self.month[month].add(idx)
            self.day[day].add(idx)
            self.day_of_week[day_of_week].add(idx)
            self.num_of_week[num_of_week].add(idx)

    def _check_holiday_structure(self, times):
        """ To check the structure of the HolidayClass

        :param list times: years or months or days or number week
        :rtype: None or Exception
        :return: in the case of exception returns the exception
        """

        if not isinstance(times, list):
            raise TypeError("an list is required")

        for time in times:
            if not isinstance(time, tuple):
                raise TypeError("a tuple is required")
            if len(time) > 5:
                raise TypeError("Target time takes at most 5 arguments"
                                " ('%d' given)" % len(time))
            if len(time) < 5:
                raise TypeError("Required argument '%s' (pos '%d')"
                                " not found" % (TIME_LABEL[len(time)], len(time)))

            self._check_time_format(TIME_LABEL, time)

    def _check_time_format(self, labels, values):
        """ To check the format of the times

        :param list labels: years or months or days or number week
        :param list values: number or the asterisk in the list
        :rtype: None or Exception
        :raises PeriodRangeError: outside the scope of the period
        :raises ParseError: not parse the day of the week
        :return: in the case of exception returns the exception
        """

        for label, value in zip(labels, values):
            if value == "*":
                continue
            if label == "day_of_week":
                if isinstance(value, string_types):
                    if value not in ORDER_WEEK:
                        raise ParseError("'%s' is not day of the week. "
                                         "character is the only '%s'" % (
                                             value, ', '.join(ORDER_WEEK)))
                elif not isinstance(value, int):
                    raise TypeError("'%s' is not an int" % value)

            if label in ["year", "month", "day", "num_of_week"]:
                if not isinstance(value, int):
                    raise TypeError("'%s' is not an int" % value)

            if isinstance(value, int):
                start, end = TIME_INFO[label]
                if not start <= value <= end:
                    raise PeriodRangeError("'%d' is outside the scope of the period "
                                           "'%s' range: '%d' to '%d'" % (
                                               value, label, start, end))

    def is_holiday(self, date):
        """ Whether holiday judges

        :param datetime date: datetime.date object
        :rtype: bool
        """
        time = [
            date.year,
            date.month,
            date.day,
            date.isoweekday(),
            _extract_week_number(date)
        ]

        target = []
        for key, data in list(zip(TIME_LABEL, time)):
            d = getattr(self, key)
            asterisk = d.get("*", set())
            s = asterisk.union(d.get(data, set()))
            target.append(list(s))

        for result in map(set, product(*target)):
            if len(result) == 1:
                return True
        return False

    def is_business_day(self, date):
        """ Whether the business day

        :rtype: bool
        """
        return not(self.is_holiday(date))


def _extract_week_number(date):
    first_date = date.replace(day=1)
    return int(date.strftime('%W')) - int(first_date.strftime('%W')) + 1
